/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "tstore.h"

int main(int argc, char ** argv)
{
  uint64_t store_size;
  int store_fd,hash_fd;
  uint8_t buf[HBS];
  char *namebuf;
  int r;
  uint64_t i;

  if (!( (argc == 2) ||
	((argc == 3)&&(strlen(argv[2])==16)) )) {
    fprintf(stderr,"Uage: %s <store> [key]\n",argv[0]);
    return 1;
  }

  store_fd=open(argv[1],O_RDONLY);
  if (store_fd==-1) {
    perror(argv[1]);
    return 1;
  }

  store_size=test_store_size(store_fd,0);
  store_error(store_size,argv[1]);

  if (asprintf(&namebuf,"%s.32h",argv[1])<4) {
    perror("Out of memory");
    return 1;
  }

  if (argc == 3) {
    memcpy(hash_key,argv[2],16);
  } else {
    int r_fd=open("/dev/urandom",O_RDONLY);
    if (r_fd == -1) {
      perror("/dev/urandom");
      memset(hash_key,0x33,16);
    } else {
      if (myread(r_fd,hash_key,16)!=16) {
	perror("/dev/urandom");
	return 1;
      }
    }
  }

  hash_fd=creat(namebuf,0666);
  if (hash_fd==-1) {
    perror(namebuf);
    return 1;
  }

  if (write(hash_fd,hash_key,16)!=16) {
    perror(namebuf);
    return 1;
  }

  for (i=1;(r=read_and_hash(store_fd,buf))==512;++i) {
    if(write(hash_fd,buf,4)!=4) {
      perror(namebuf);
      return 1;
    }
  }

  switch(r) {
  default:
    fprintf(stderr,"%s: Incomplete read\n",argv[1]);
    return 1;
  case -1:
    perror(argv[1]);
    return 1;
  case 0:
    break;
  }
  if (i != store_size) {
    fprintf(stderr,"%s: Has %s as we read it\n",
	    argv[1],(i>store_size)?"grown":"shrunk");
    return 1;
  }

  return 0;
}
