/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "tstore.h"

uint64_t size_in_blocks(int fd)
{
  struct stat stbuf;
  if (fstat(fd,&stbuf)) return -1;
  if (stbuf.st_size % 512) return 0;
  return stbuf.st_size/512;
}

uint64_t test_store_size(int fd, int p)
{
  uint8_t buf[512];
  uint64_t store_size=size_in_blocks(fd);
  if (store_size<1) return 0;

  switch (p?pread(fd,buf,512,0):read(fd,buf,512)) {
  case -1:
    return -1;
  default:
    return 0;
  case 512:
    if (memcmp(buf,"TSTORE5",8))
      return 0;
  }

#if 0
  if ((GET_BNUM(buf+8)!=store_size)||
      (GET_BNUM(buf+13)>=store_size)) return 0;
#endif

  return store_size;
}

void store_error(uint64_t size, char *name)
{
  switch(size) {
  case -1:
    perror(name);
    exit(1);
  case 0:
    fprintf(stderr,"%s: Not a valid store\n",name);
    exit(1);
  }
}

ssize_t myread(int fd, void *buf, size_t count)
{
  ssize_t r,total=0;
  memset(buf,0,count);
  while (count) {
    r=read(fd,buf,count);
    if (r<1) return total?total:r;
    total+=r;
    buf=(((char*)buf)+r);
    count-=r;
  }
  return total;
}

int filelevels(uint64_t size)
{
  int levels=0;
  uint64_t maxsize=512;
  while(size>maxsize) {
    levels++;
    maxsize*=102;
  }
  return levels;
}

int seqblock(uint8_t *buf)
{
  uint64_t v;
  int i;
  if (buf[510]||buf[511]||(buf[0]>=0xC0)) return 0;
  v=GET_BNUM(buf);
  for (i=1;i<102;++i)
    if (GET_BNUM(buf+5*i)!=v+i) return 0;
  return 1;
}

void get_block(uint64_t n, uint8_t *buf, char *store_name, int store_fd)
{
  if (n) {
    if (n&(3LL<<38)) {
      int i;
      n-=(1LL<<38);
      for (i=0;i<102;++i)
	PUT_BNUM(buf+5*i,n+i);
      buf[510]=buf[511]=0;
    } else {
      switch(pread(store_fd,buf,512,512*n)) {
      case -1:
	perror(store_name);
	exit(1);
      default:
	fprintf(stderr,"%s: short pread (block %lld)\n",store_name,n);
	exit(1);
      case 512:
	break;
      }
    }
  } else
    memset(buf,0,512);
}
